import java.util.*;
class SortAll {
    public static void main (String[] args) {
    	int   seed = 3885;
    	int[] size = {1000, 10000, 50000, 500000};
    	int[] data;
    	long start, end;

    	System.out.println(" size:  bubble  insert Array.sort   heap1   heap2   quick");
    	for (int i=0; i<size.length; i++) {
	    System.out.print(format(size[i], 6));

	    data = make_array(size[i], seed);
	    start = System.currentTimeMillis();
	    bubble_sort(data);
	    end = System.currentTimeMillis();
	    System.out.print(format(end-start, 8));

	    data = make_array(size[i], seed);
	    start = System.currentTimeMillis();
	    insert_sort(data);
	    end = System.currentTimeMillis();
	    System.out.print(format(end-start, 8));

	    data = make_array(size[i], seed);
	    start = System.currentTimeMillis();
	    Arrays.sort(data); // improved quick sort
	    end = System.currentTimeMillis();
	    System.out.print(format(end-start, 11));

	    data = make_array(size[i], seed);
	    start = System.currentTimeMillis();
	    heap_sort(data);
	    end = System.currentTimeMillis();
	    System.out.print(format(end-start, 8));

	    data = make_array(size[i], seed);
	    start = System.currentTimeMillis();
	    heap_sort2(data);
	    end = System.currentTimeMillis();
	    System.out.print(format(end-start, 8));

	    data = make_array(size[i], seed);
	    start = System.currentTimeMillis();
	    quick_sort(data, 0, data.length-1);
	    end = System.currentTimeMillis();
	    System.out.println(format(end-start, 8));
    	}
    }

    public static String format (long num, int len) {
    	String num_str = Long.toString(num);
    	int pad_num = len - num_str.length();

    	for (int i=0; i<pad_num; i++) {
    	    num_str = " "+num_str;
    	} 
    	return num_str;
    }
    
    static void swap (int[] data, int i, int j) {
    	int tmp=data[i]; data[i]=data[j]; data[j]=tmp;
    }

    static int[] make_array (int size, long seed) {
    	Random random = new Random(seed);
    	int[]  data = new int[size];

	for (int i=0; i<data.length; i++) {
    	    // 0`size-1 ͈̔͂Ń_Ƀf[^𐶐
	    data[i] = Math.abs(random.nextInt()) % size;
	}
    	return data;
    }

    static void print_array (int[] data) {
	for (int i=0; i<data.length; i++) {
    	    System.out.println(data[i]);
	}
    }

    static boolean is_sorted_array(int[] data) {
	for (int i=0; i<data.length-1; i++) {
	    if (data[i]>data[i+1]) {
		return false;
	    }
	}
	return true;
    }

    static void bubble_sort(int[] data) {
	for (int i=data.length-1; i>0; i--) {
	    for (int j=0; j<i; j++) {
		if (data[j] > data[j+1]) {
    	    	    swap(data, j, j+1);
		}
	    }
	}
    }

    static void insert_sort(int data[])
    {
	for (int i=1; i<data.length; i++) {
    	    for (int j=i; j>0 && data[j-1]>data[j]; j--) {
    	    	swap(data, j-1, j);
	    }
	}
    }

    static int left   (int i) { return 2*i+1; }
    static int right  (int i) { return 2*i+2; }
    static int parent (int i) { return (i-1)/2; }

/* q[v̍m[h from݂̂̕sD(E̕؂
   q[vƉ肷)DC͉ɓ`dD*/
    static void downheap (int[] data, int from, int to) {
    	for (int i=from, j; left(i)<=to; i=j) {
    	    j=left(i);
	    if (right(i)<=to && data[left(i)]<=data[right(i)]) {
		j=right(i);
	    }
    	    if (data[j]>data[i]) {
    	    	swap(data, i, j);
	    } else {
		return;
	    }
	}
    }

    // q[vɂ͍ɑ傫f[^悤ɃCT[g
    static void heap_insert (int[] data, int n) {
    	// eCړ_ŖԂ͌JԂD
    	for (int i=n; i>0 && data[i]>data[parent(i)]; i=parent(i)) {
	    swap(data, i, parent(i)); // eqœꊷ
    	}
    }

    static void heap_sort (int[] data) {
	/* ̃xCUăq[v */
	for (int i=1; i<data.length; i++) {
    	    heap_insert(data, i);
	}

	/* ővf data[0] 𔲂Ă */
	for (int i=data.length-1; i>0; i--) {
    	    swap(data, 0, i);
	    downheap(data, 0, i-1);
	}
    }

    static void heap_sort2 (int[] data) {
	/* ̃xCUăq[v */
	for (int i=(data.length-1)/2; i>=0; i--) {
    	    downheap(data, i, data.length-1);
	}

	/* ővf data[0] 𔲂Ă */
	for (int i=data.length-1; i>0; i--) {
    	    swap(data, 0, i);
	    downheap(data, 0, i-1);
	}
    }

    static void quick_sort(int[] data, int from, int to) {
    	if (from>=to) return;
    	int pivot=data[(from+to)/2];
	int left=from, right=to;

    	while (left<=right) {
	    while (data[left]<pivot)  left++; // pivotȏT
	    while (data[right]>pivot) right--; // pivotȉT
    	    if (left<=right) {
    	    	swap(data, left, right);
    	    	left++; right--;
    	    }
    	}
    	// System.out.println("left="+left+", right="+right);
	quick_sort(data, from, right);
	quick_sort(data, left, to);
    }
}
